// Utility Functions
function fetchAndStoreAuthToken() {
    let authToken = sessionStorage.getItem('authToken');
    let expirationTime = sessionStorage.getItem('authTokenExpiration');

    if (authToken && expirationTime) {
        const now = Date.now();
        if (now < parseInt(expirationTime, 10)) {
            return Promise.resolve(authToken);
        }
    }

    return $.post("/_services/auth/token?client_id=bkpv").then(function (token, status, xhr) {
        const expiresIn = xhr.getResponseHeader('Expires-In');
        const dateHeader = xhr.getResponseHeader('Date');
        const serverTime = dateHeader ? new Date(dateHeader).getTime() : Date.now();
        const expirationTime = serverTime + parseInt(expiresIn, 10) * 1000;

        sessionStorage.setItem('authToken', token);
        sessionStorage.setItem('authTokenExpiration', expirationTime.toString());
        return token;
    }).fail(function (error) {
        console.log("Error fetching authToken", error);
        return null;
    });
};

(function (webapi, $) {
    /**
     * Function to send AJAX requests with token handling.
     * @param {string} url - The URL to which the request is sent.
     * @param {string} method - The HTTP method to use for the request (e.g., "GET", "POST", "PUT").
     * @param {Object} data - The data to send to the server.
     * @param {Object} options - Additional AJAX options (headers, contentType, processData, etc.).
     * @param {Function} successCallback - Callback function that is called if the request succeeds.
     * @param {Function} errorCallback - Callback function that is called if the request fails.
     * @param {boolean} useToken - Whether to fetch and use a token for this request.
     */
    function sendAjaxRequest(url, method, data, options = {}, successCallback, errorCallback, useToken = true, isExternalApi = false) {
        var deferredAjax = $.Deferred();

        // Function to execute the AJAX request
        function executeRequest(token) {
            // Set headers based on whether the request is for an external API
            var headers = options.headers || {};
            if (useToken && token) {
                if (isExternalApi) {
                    headers['Authorization'] = 'Bearer ' + token;
                } else {
                    headers['__RequestVerificationToken'] = token;
                }
            }

            var ajaxOptions = {
                url: url,
                type: method,
                headers: headers,
                success: function (response, textStatus, jqXHR) {
                    if (typeof successCallback === 'function') {
                        successCallback(response, textStatus, jqXHR);
                    }
                    deferredAjax.resolve(response, textStatus, jqXHR);
                },
                error: function (jqXHR, textStatus, errorThrown) {
                    if (typeof errorCallback === 'function') {
                        errorCallback(jqXHR, textStatus, errorThrown);
                    }
                    deferredAjax.reject(jqXHR, textStatus, errorThrown);
                }
            };

            // Handle binary data
            if (options.contentType === 'application/octet-stream') {
                ajaxOptions.contentType = options.contentType;
                ajaxOptions.processData = false;
                ajaxOptions.data = data;
            } else {
                ajaxOptions.contentType = 'application/json';
                ajaxOptions.data = JSON.stringify(data);
            }

            $.ajax(ajaxOptions);
        }

        if (useToken) {
            // Fetch the token; the method could be different based on whether it is an external API or not
            var tokenPromise = isExternalApi ? fetchAndStoreAuthToken() : shell.getTokenDeferred();
            tokenPromise.done(function (token) {

                executeRequest(token);
            }).fail(function (jqXHR, textStatus, errorThrown) {
                console.error("Failed to fetch token:", textStatus, errorThrown);
                if (typeof errorCallback === 'function') {
                    errorCallback(jqXHR, textStatus, errorThrown);
                }
                deferredAjax.reject(jqXHR, textStatus, errorThrown);
            });
        } else {
            // Execute request without a token
            executeRequest();  // No token is passed here
        }

        return deferredAjax.promise();
    }

    // Expose the sendAjaxRequest function to the webapi object
    webapi.sendAjaxRequest = sendAjaxRequest;
})(window.webapi = window.webapi || {}, jQuery);

function waitForRadiosAndRun(callback) {
    var checkExist = setInterval(function () {
        if ($('tr[role="group"]').find('.boolean-radio').length) {
            clearInterval(checkExist);
            callback();
        }
    }, 100); // Check every 100ms
};

function patchSSN(ssn, applicationId, ssnFieldLogicalName, onSuccess, onError) {
    const url = `/_api/bkpv_consumeraccountapplications(${applicationId})`;
    const dataPayload = {};
    dataPayload[ssnFieldLogicalName] = ssn;
    const headers = {}; // Add any headers you need here, but the token header will be handled by sendAjaxRequest

    // Call the refactored sendAjaxRequest with token handling
    webapi.sendAjaxRequest(
        url,                // API endpoint
        'PATCH',            // Method type
        dataPayload,        // Data to be sent
        headers,            // Additional headers
        onSuccess,          // Success callback
        function (jqXHR, textStatus, errorThrown) { // Error callback
            console.error('Error updating SSN:', textStatus, errorThrown);
            if (typeof onError === 'function') {
                onError('Error updating SSN: ' + errorThrown);
            }
        },
        true                // Use token
    );
};

// Function to insert form instructions below a specific element
function formInstructionsBelow(content) {
    $('#WebFormPanel').after(content);
};

function showAlert(message, type) {
    // This function shows an alert to the user. You can style it according to your application's theme.
    // Here, we assume you have a div with id 'alert-box' to show messages.
    const alertBox = $('#alert-box');
    alertBox.text(message).addClass(type).fadeIn(200).delay(4000).fadeOut(500);

    // Clear any previous classes and ensure the correct one is applied
    alertBox.removeClass('success error').addClass(type);
};

function styleYesNo() {
    // Find each group of radio buttons
    $('tr[role="group"]').each(function () {
        var $group = $(this).find('.boolean-radio');
        var $radios = $group.find('input[type="radio"]');

        // Create a new btn-group div
        var $btnGroup = $('<div class="btn-group" role="group" aria-label="Yes or No"></div>');
        // Remove the visually-hidden span if it exists within the label

        // Convert each radio and its label into a button
        $radios.each(function () {
            var $radio = $(this);
            var $label = $radio.next('label');
            var $tooltip = $label.find('span.visually-hidden');
            $tooltip.remove();

            var labelText = $label.text().trim();

            // Hide the original radio and update the label
            $radio.addClass('btn-check').attr('autocomplete', 'off');

            // Check if the label is "Yes" or "No" and add the corresponding class
            if (labelText === 'No') {
                $label.addClass('btn btn-default').attr('for', $radio.attr('id'));
            } else if (labelText === 'Yes') {
                $label.addClass('btn btn-secondary').attr('for', $radio.attr('id'));
            }

            // Append the radio and label to the new btn-group
            $btnGroup.append($radio).append($label);

            // Update the label text to properly format it within the button
            $label.contents().filter(function () {
                return this.nodeType === 3; // Node.TEXT_NODE
            }).remove();
            $label.append(labelText);
        });

        // Replace the span with the new btn-group div
        $group.replaceWith($btnGroup);
    });
};


function initializeAddressAutocomplete(streetFieldId, cityFieldId, stateFieldId, zipFieldId) {
    $(document).ready(function () {
        var streetInput = $('#' + streetFieldId);

        // Setup autocomplete
        streetInput.autocomplete({
            minLength: 3,
            source: function (request, response) {
                // Define the URL with the query term
                var url = '/portal/addressAutocomplete?query=' + encodeURIComponent(request.term);

                // Use the generalized sendAjaxRequest function
                window.webapi.sendAjaxRequest(
                    url,
                    'GET',
                    {}, // No data needs to be sent for this GET request
                    {}, // Headers will be managed by sendAjaxRequest depending on the token requirement
                    function (data) { // Success callback

                        response(data.results);
                    },
                    function (error) { // Error callback
                        console.error('Error fetching autocomplete data:', error);
                        response([]); // Respond with empty data on error to handle UI gracefully
                    },
                    true, // Use token
                    true // Not an external API in this context
                );
            },
            select: function (event, ui) {
                event.preventDefault();  // Prevent the default behavior
                populateAddressFields(ui.item, streetFieldId, cityFieldId, stateFieldId, zipFieldId);
                return false;  // Prevent other default actions
            }
        }).autocomplete("instance")._renderItem = function (ul, item) {
            ul.addClass("list-group");
            return $("<li>")
                .addClass("list-group-item list-group-item-action")
                .append("<div>" + item.address.freeformAddress + "</div>")
                .appendTo(ul);
        };
    });
};


function populateAddressFields(item, streetFieldId, cityFieldId, stateFieldId, zipFieldId) {


    // Check if the street address is defined
    if (item.address && item.address.streetNumber && item.address.streetName) {
        $('#' + streetFieldId).val(item.address.streetNumber + ' ' + item.address.streetName);
    } else {
        console.log("Error: Street address data is missing or undefined.");
    }

    // Populate the city
    $('#' + cityFieldId).val(item.address.municipality);

    // Populate the state
    var stateFullName = item.address.countrySubdivisionName;
    $('#' + stateFieldId).find('option').each(function () {
        if ($(this).text() === stateFullName) {
            $(this).parent().val($(this).val()).trigger('change');
        }
    });

    // Populate the zip code
    $('#' + zipFieldId).val(item.address.postalCode);
};

function fetchAuthTokenAndExecute(callback) {
    fetchAndStoreAuthToken().then(authToken => {
        if (!authToken) {
            console.error('No auth token available');
            showAlert('Authentication failed. Please try again.', 'error');
        } else {
            callback(authToken);
        }
    }).catch(error => {
        console.error('Error in token fetching: ', error);
        showAlert('Authentication error. Unable to fetch valid token.', 'error');
    });
};

// Plaid Functionality
function getPlaidToken() {

    const url = "/portal/getPlaidToken";
    const headers = { 'Content-Type': 'application/json' };

    webapi.sendAjaxRequest(url, "GET", null, headers,
        (response) => {
            const linkToken = response.link_token;
            initializePlaid(linkToken);
        },
        (error) => console.log("Failed to fetch Plaid token", error),
        true,
        true
    );
};

function initializePlaid(linkToken) {
    let handler = Plaid.create({
        token: linkToken,
        onSuccess: onSuccessHandler,
        onLoad: () => { },
        onExit: onExitHandler,
        onEvent: (eventName, metadata) => { },
    });

    handler.open();
};

function onExitHandler() {
    if ($('#exitWarning').length) { }
    else {
        var content = "<div id=\"exitWarning\" class=\"alert alert-warning\" role=\"alert\"><p>We did not receive a link to your external account, hit next to proceed without funding or attempt to connect again.</p></div>";
        $('#bkpv_achfundingamount').val(0);
        $('#bkpv_fundingamount').val(0);
    }
    $('#WebFormPanel').after(content);
    $('#NextButton').fadeIn();
};

function onSuccessHandler(public_token, metadata) {

    const recordId = $("#EntityFormView_EntityID").val(); // Fetching the 'id' parameter
    const apiEndpoint = '/portal/createplaiditem';
    const data = {
        public_token: public_token,
        metadata: metadata,
        record_id: recordId
    };
    const headers = {
        'Content-Type': 'application/json'
    };

    webapi.sendAjaxRequest(apiEndpoint, 'POST', data, headers,
        (response) => $('#NextButton').trigger('click'),
        (error) => console.log("Error processing Plaid success: ", error),
        true,
        true
    );
};

function createAndAppendPlaidLinkButton(insertAfterId) {
    // Hide the Next Button
    $('#NextButton').hide();

    // Create the button
    var $button = $('<button/>', {
        text: 'Connect Your Bank Account', // Button text
        id: 'plaidLink',
        class: 'btn btn-primary', // Button class
        click: function (event) { // Attach click event handler
            event.preventDefault();
            getPlaidToken();
        }
    });

    // Wrap the button in a div with class 'input-group-append'
    var $wrapper = $('<div/>', {
        class: 'input-group-append'
    }).append($button);

    // Insert the wrapped button after the specified element
    $wrapper.insertAfter('#' + insertAfterId);
};

function checkboxFix(dataName) {
    var selector = 'div[data-name="' + dataName + '"]';
    $(selector).find('.checkbox-cell').each(function () {
        var $cell = $(this);
        var $checkbox = $cell.find('input[type="checkbox"]');
        var $label = $cell.find('label');

        // Check if the checkbox and label are already inside a form-check wrapper
        if (!$cell.find('.form-check').length) {
            // Create new divs for Bootstrap grid structure
            var $checkboxP = $('<p class="ps-4"></p>');

            // Move checkbox into the checkbox div and add form-check-input class
            $checkboxP.append($checkbox.addClass('form-check-input col-2 me-3'));
            $label.append('<span class="text-danger">*</span>');
            // Move label into the label div and add form-check-label class
            $checkboxP.append($label.addClass('form-check-label col-10 d-sm-inline-block'));

            // Replace the contents of the cell with this new row div
            $cell.empty().append($checkboxP);
        } else {
            // Ensure classes are correctly applied if the structure already exists
            $cell.find('.form-check').addClass('row');
            $checkbox.addClass('col-2 form-check-input');
            $label.addClass('col-10 form-check-label');
        }
    });
};



// Function to create an accordion item using jQuery
function createDisclosureAccordionItem(id, title, content, agreeContent, url) {
    var accordionItem = $('<div class="accordion-item ps-4" style="padding-top:10px;padding-bottom:10px;border:none;"></div>');
    var accordionHeader = $(`
      <p class="accordion-header" id="heading${id}P">
     
     <input class="form-check-input col-2 me-3" type="checkbox" id="heading${id}">
     <label class="form-check-label col-10" for="heading${id}">${agreeContent} <a style="padding:0px;margin-left:0px;" class="link-primary" data-bs-toggle="collapse" data-bs-target="#collapse${id}" aria-expanded="false" aria-controls="collapse${id}">${title}</a><span class="text-danger">*</span></label>
      </p>
  `);
    var accordionCollapse = $(`
      <div id="collapse${id}" class="accordion-collapse collapse" aria-labelledby="heading${id}" data-bs-parent="#disclosureAccordion">
          <div class="accordion-body"><h1>${title}</h1><a href="${url}" class="btn-secondary">Download PDF</a><div class="mt-3">${content}</div></div>
      </div>
  `);

    // Append the header and collapse sections to the accordion item
    accordionItem.append(accordionHeader);
    accordionItem.append(accordionCollapse);

    return accordionItem;
};
function styleListView(divId) {
    var container = $('#' + divId); // Select the div by ID
    var menu = container.find('.dropdown-menu');
    var buttonGroup = $('<div class="btn-group btn-group-lg"></div>');

    menu.find('li a').each(function () {
        var text = $(this).text().trim(); // Trim whitespace
        var iconClass;

        // Determine the appropriate icon based on the link text
        switch (text.toLowerCase()) { // Convert text to lowercase for case-insensitive comparison
            case 'edit': iconClass = 'btn px-1 fa fa-edit'; break;
            case 'delete': iconClass = 'btn px-1 fa fa-trash'; break;
            case 'view': iconClass = 'btn px-1 fa fa-eye'; break;
            default: iconClass = 'btn px-1 fa fa-question-circle'; break; // Default icon if no match
        }


        // Clear the text and append the icon to the link
        $(this).empty().addClass(iconClass);
        // Add the 'btn' and 'btn-info' classes to the link for styling

        // Append the modified link to the button group
        buttonGroup.append($(this));
    });

    // Replace the dropdown with the new button group
    container.find('.dropdown').replaceWith(buttonGroup);
};


function waitForElement(dataEntity, divId) {
    var interval = setInterval(function () {
        var $element = $("tr[data-entity='" + dataEntity + "']");
        if ($element.length > 0) {
            clearInterval(interval);

            styleListView(divId); // Call your function to style the list or do other work
        }
    }, 100); // Checks every 500ms
};

function consumerSubmit(id) {
    // Updated API endpoint for POST request
    const apiEndpoint = '/portal/submitConsumerApplication';
    const requestData = {}; // Prepare your request data here

    requestData["id"] = id;
    const headers = { 'Content-Type': 'application/json' };

    webapi.sendAjaxRequest(apiEndpoint, 'POST', requestData, headers,
        function (response) {

            // Check if response.message is "Approved"
            if (response.message === "Approved") {
                approved(response.message, response.accountNumber,"RegistrationDisclosure.aspx");
            } else {
                review();
            }
        },
        function (xhr, status, error) {
            console.error('POST request failed', xhr, status, error);
            // Optionally call review() on error or handle differently
            review();
        },
        true,
        true
    );
};
function businessSubmit(id) {
    // Updated API endpoint for POST request
    const apiEndpoint = '/portal/submitBusinessApplication';
    const requestData = {}; // Prepare your request data here

    requestData["id"] = id;
    const headers = { 'Content-Type': 'application/json' };

    webapi.sendAjaxRequest(apiEndpoint, 'POST', requestData, headers,
        function (response) {

            // Check if response.message is "Approved"
            if (response.message === "Approved") {
                approved(response.message, response.accountNumber,"BusinessEnroll.aspx");
            } else {
                review();
            }
        },
        function (xhr, status, error) {
            console.error('POST request failed', xhr, status, error);
            // Optionally call review() on error or handle differently
            review();
        },
        true,
        true
    );
};
function approved(message, account, path) {
    $('#statuscode').text(message);
    $('#bkpv_accountnumber_label').parent().fadeIn();
    $('#bkpv_accountnumber').parent().fadeIn();
    $('#bkpv_accountnumber').val(account);
    $('#bkpv_accountnumber').closest('.form-control-cell').find('.text.muted').remove();
    // Update or insert #approvalMessage
    if ($('#approvalMessage').length) {
        // Update existing content
        $('#approvalMessage').html(`<p>Your application has been approved</p><p style="font-size: 16px;">Would you like to register for online banking?</br>Navigate to <a href="https://secure.myvirtualbranch.com/BankProvOnline/${path}">Our Registration Page</a> to continue</p></div>`);
    }

    // Open a new tab after 10 seconds
    //setTimeout(function() {
    //    window.open('https://secure.myvirtualbranch.com/BankProvOnline/Register.aspx', '_blank');
    //}, 10000);
};
function previouslyApproved(path) {

    $('#bkpv_accountnumber_label').parent().fadeIn();
    $('#bkpv_accountnumber').parent().fadeIn();

    $('#bkpv_accountnumber').closest('.form-control-cell').find('.text.muted').remove();

    $('#approvalMessage').html(`<p>Your application has been approved</p><p style="font-size: 16px;">Would you like to register for online banking?</br>Navigate to <a href="https://secure.myvirtualbranch.com/BankProvOnline/${path}">Our Registration Page</a> to continue</p></div>`);



    // Open a new tab after 10 seconds
    //setTimeout(function() {
    //    window.open('https://secure.myvirtualbranch.com/BankProvOnline/Register.aspx', '_blank');
    //}, 10000);
};
function review() {
    $('#statuscode').text('Review');
    // Update or insert #approvalMessage
    if ($('#approvalMessage').length) {
        // Update existing content
        $('#approvalMessage').html('<p>Your application is being reviewed</p>' +
            '<p style="font-size: 16px;">Please Check back later for the outcome</p>');
    }
};

function companyNameStep() {
    // Code specific to the "companyName" step
    $('#loadingSpinner').hide();
    $('#EntityFormView').show();
};

function businessTypeStep() {
    // Code specific to the "businessType" step
    // Show loading spinner and hide form initially
    $('#loadingSpinner').show();
    $('#EntityFormView').hide();

    // Fetch and populate types
    $.ajax({
        type: "GET",
        url: "/_api/bkpv_restrictedcustomertypeses?$select=bkpv_name,bkpv_description",
        success: function (response) {
            populateTypes(response.value);
            $('#loadingSpinner').fadeOut();
            $('#EntityFormView').fadeIn();
        },
        error: function (error) {
            console.error("Error fetching types:", error);
            $('#loadingSpinner').fadeOut();
        }
    });

    // Populate dynamic checkboxes
    function populateTypes(types) {
        var typesHtml = $('<div class="row"></div>');
        types.forEach(function (type) {
            var checkboxHtml = `
            <div class="col-md-6">
                <input type="checkbox" id="type_${type.bkpv_name}" name="type" value="${type.bkpv_name}">
                <label for="type_${type.bkpv_name}" title="${type.bkpv_description}">${type.bkpv_name}</label>
            </div>
        `;
            typesHtml.append(checkboxHtml);
        });
        $('.tab-column').before(typesHtml);
    }

};

function ConvertSelectToAutocomplete(selectName, selectPlaceholder = "") {
    var selectElement = $("#" + selectName);
    var autoCompleteElementId = selectName + "-autocomplete";
    var autoCompleteDatasetId = selectName + "-data";
    var options = selectElement.find("option").map(function () {
        return `<option data-value="${$(this).val()}" value="${$(this).text()}"/>`;
    }).get().join('');

    selectElement.after(`
    <input type="text" id="${autoCompleteElementId}" class="${selectElement.attr('class').replace("form-select", "form-control")} form-control" 
           list="${autoCompleteDatasetId}" placeholder="${selectPlaceholder}" 
           ${selectElement.attr("readonly") ? 'readonly' : ''}>
    <datalist id="${autoCompleteDatasetId}">${options}</datalist>
`);
    selectElement.hide();

    $("#" + autoCompleteElementId).on("change", function () {
        var value = $("#" + autoCompleteDatasetId + " option[value='" + $(this).val() + "']").data("value");
        selectElement.val(value);
        if (value === undefined) $(this).val("");
    });
};

function formatEINSSN() {
    var label = $('#bkpv_ein_label').text().trim(); // Get the label text and trim any extra whitespace
    var inputVal = $('#bkpv_ein').val().replace(/\D/g, ''); // Remove non-digit characters

    if (label === 'Social Security Number') {
        // Format as SSN (XXX-XX-XXXX) progressively
        inputVal = inputVal.substring(0, 9); // Limit SSN to 9 digits
        var formattedSSN = inputVal
            .replace(/(\d{3})/, '$1-') // Add dash after first 3 digits
            .replace(/(\d{3})-(\d{2})/, '$1-$2-'); // Add dash after next 2 digits
        $('#bkpv_ein').val(formattedSSN);
    } else if (label === 'Employer Identification Number') {
        // Format as EIN (XX-XXXXXXX) progressively
        inputVal = inputVal.substring(0, 9); // Limit EIN to 9 digits
        var formattedEIN = inputVal
            .replace(/(\d{2})/, '$1-') // Add dash after first 2 digits
            .replace(/(\d{2})-(\d{7})/, '$1-$2'); // Ensure dash remains correctly after 2 digits
        $('#bkpv_ein').val(formattedEIN);
    }
};

function adjustEINFields() {
    var entityType = $('#bkpv_entitytype').val();
    var einOverrideChecked = $('#bkpv_einoverride').is(':checked');
    var $originalInput = $('#bkpv_ein').attr('type', 'text'); // Make sure the input is visible and editable
    var $dbaField = $('#bkpv_dba').closest('.cell');
    var $einLabel = $('#bkpv_ein_label');
    var $einCell = $einLabel.closest('.table-info').addClass('required');

    if (entityType === '100000004' && !einOverrideChecked) {
        $einLabel.html('Social Security Number');
        $originalInput.attr({
            'aria-label': 'Social Security Number of Owner',
            'title': 'Social Security Number is a required field.'
        });
        $dbaField.fadeIn();
        $('#bkpv_einoverride').closest('.cell').fadeIn();
    } else {
        $einLabel.html('Employer Identification Number');
        $originalInput.attr({
            'aria-label': 'Employer Identification Number',
            'title': 'EIN is a required field.'
        });
        if (entityType !== '100000004') {
            $dbaField.fadeOut();
            $('#bkpv_einoverride').closest('.cell').hide();
        } else {
            $dbaField.fadeIn();
            $('#bkpv_einoverride').closest('.cell').fadeIn();
        }
    }

};

function companyDetailsStep() {
    $('#loadingSpinner').hide();
    $('#EntityFormView').show();
    $('#bkpv_ein').on('input', formatEINSSN);
    adjustEINFields(); // Adjust fields initially
    ConvertSelectToAutocomplete("bkpv_naisc", "Type to search...");
    $('#bkpv_entitytype, #bkpv_einoverride').change(adjustEINFields);
    // Styling the checkbox as part of initialization
    $("#bkpv_einoverride, #bkpv_einoverride_label").wrapAll('<div class="form-check"></div>');
    $("#bkpv_einoverride").addClass('form-check-input');
    $("#bkpv_einoverride_label").addClass('form-check-label');
};



function businessAddressStep() {
    initializeAddressAutocomplete(
        'bkpv_streetaddress', // ID of the street address input
        'bkpv_city',          // ID of the city input
        'bkpv_physicaladdressstate', // ID of the state select
        'bkpv_zipcode'       // ID of the zipcode input
    );
};

function setupCurrencyInput(fieldId) {
    // Find the input element using the provided ID
    var inputElement = document.getElementById(fieldId);
    if (!inputElement) {
        console.error('Input field not found');
        return;
    }

    // Set the input type to 'number'
    inputElement.setAttribute('type', 'number');

    // Add Bootstrap classes for currency input
    inputElement.classList.add('form-control', 'currency-input');

    // Check if the parent node is an input group; if not, create one
    var parentDiv = inputElement.parentNode;
    if (!parentDiv.classList.contains('input-group')) {
        var inputGroup = document.createElement('div');
        inputGroup.className = 'input-group mb-3';

        // Insert the input group in place of the input element
        parentDiv.replaceChild(inputGroup, inputElement);
        inputGroup.appendChild(inputElement);

        // Add the currency symbol
        addCurrencySymbol(inputGroup, inputElement);
    } else if (!parentDiv.querySelector('.input-group-text')) {
        // If it's already an input group but no currency symbol, add it
        addCurrencySymbol(parentDiv, inputElement);
    }
};

function addCurrencySymbol(inputGroup, inputElement) {
    var span = document.createElement('span');
    span.className = 'input-group-text';
    span.textContent = '$'; // Currency symbol
    inputGroup.insertBefore(span, inputElement);
};

function uploadFile(file, description, bkpv_type, appId) {
    var data = {
        bkpv_type: bkpv_type,
        bkpv_description: description,
        statuscode: 810850001,
        "bkpv_Application@odata.bind": "/bkpv_businessaccountapplications(" + appId + ")"
    };

    return new Promise((resolve, reject) => {
        webapi.sendAjaxRequest(
            '/_api/bkpv_documentations',
            'POST',
            data,
            {},
            function (response, textStatus, jqXHR) {

                var locationUrl = jqXHR.getResponseHeader('Location');
                if (locationUrl) {
                    sendFileData(locationUrl + '/bkpv_file', file)
                        .then(() => resolve(response))
                        .catch(error => reject(error));
                } else {
                    console.error('Location header is missing.');
                    reject('Location header is missing.');
                }
            },
            function (error) {
                console.log('Upload failed:', error);
                reject('Failed to upload file.');
            }
        );
    });
};

function sendFileData(url, file) {
    return new Promise((resolve, reject) => {
        var reader = new FileReader();
        reader.onload = function (e) {
            var fileData = new Uint8Array(e.target.result);

            webapi.sendAjaxRequest(
                url + "?x-ms-file-name=" + file.name,
                'PUT',
                fileData,
                {
                    contentType: 'application/octet-stream',
                    processData: false
                },
                function (response, textStatus, jqXHR) {

                    resolve(response);
                },
                function (error) {
                    console.log('Upload failed:', error);
                    reject('Failed to upload file data.');
                }
            );
        };
        reader.readAsArrayBuffer(file);
    });
};


function showDescriptionInput(link) {
    var descriptionInput = link.closest('.mb-3').querySelector('.descriptionInput');
    if (descriptionInput.hidden === true) {
        descriptionInput.hidden = false;
    } else {
        descriptionInput.hidden = true;
    }
};